package com.itcast.bulls.stock.deal.service;

import com.itcast.bulls.stock.common.encrypt.EncryptUtil;
import com.itcast.bulls.stock.common.utils.GlobalConstants;
import com.itcast.bulls.stock.deal.dao.biz.IStockUserDao;
import com.itcast.bulls.stock.deal.dao.biz.ITradeAccountDao;
import com.itcast.bulls.stock.entity.user.TradeAccount;
import com.itcast.bulls.stock.entity.user.TradeUser;
import com.itcast.trade.stock.struct.grpc.trade.LoginRequest;
import com.itcast.trade.stock.struct.grpc.trade.LoginResponse;
import com.itcast.trade.stock.struct.grpc.trade.StockUserServiceGrpc;
import io.grpc.stub.StreamObserver;
import lombok.extern.log4j.Log4j2;
import net.devh.boot.grpc.server.service.GrpcService;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.Date;

/**
 * <p>Description: </p>
 * @date 
 * @author 
 * @version 1.0
 * <p>Copyright:Copyright(c)2020</p>
 */
@GrpcService
@Log4j2
public class GrpcStockUserService extends StockUserServiceGrpc.StockUserServiceImplBase {

    /**
     * 用户管理的数据层接口
     */
    @Autowired
    private IStockUserDao stockUserDao;

    /**
     * 交易账号的数据层接口
     */
    @Autowired
    private ITradeAccountDao tradeAccountDao;

    /**
     * 用户登录接口处理
     * @param request
     * @param responseObserver
     */
    @Override
    public void userLogin(LoginRequest request, StreamObserver<LoginResponse> responseObserver) {
        LoginResponse response = null;
        try {
            // 1.根据用户编号, 查找获取用户对象信息(做校验判断, 如果用户不存在, 则抛出异常提示)
            TradeUser tradeUser = stockUserDao.getByUserNo(request.getUserNo());
            if(null == tradeUser) {
                response = LoginResponse.newBuilder().setStatus(GlobalConstants.STATUS_ERROR).setMessage("用户不存在!").build();
            }else {
                // 2. 用户密码校验, 通过加密方式判断
                String encryptPassword = EncryptUtil.encryptSigned(request.getUserPwd());
                boolean pwdMatch = tradeUser.getUserPwd().equals(encryptPassword);
                if(!pwdMatch) {
                    // 密码不一致, 给出错误消息提示
                    response = LoginResponse.newBuilder().setStatus(GlobalConstants.STATUS_ERROR).setMessage("用户密码错误!").build();
                }else {
                    // 3. 返回用户的登录成功信息(返回用户账号信息)
                    TradeAccount tradeAccount = tradeAccountDao.getByUserId(tradeUser.getId());
                    // 4. 更新用户登录的相关信息
                    stockUserDao.updateUserLogin(tradeUser.getId(), new Date());
                    response = LoginResponse.newBuilder().setStatus(GlobalConstants.STATUS_OK)
                            .setUserId(tradeUser.getId())
                            .setAccountId(tradeAccount.getId())
                            .setMessage("用户登录成功!").build();
                    log.info("用户{}登录成功!", tradeUser.getName());
                }

            }


        }catch(Exception e){
            log.error(e.getMessage(), e);
            response = LoginResponse.newBuilder().setStatus(GlobalConstants.STATUS_ERROR).setMessage(e.getMessage()).build();
        }
        responseObserver.onNext(response);
        responseObserver.onCompleted();


    }
}
