package com.itcast.bulls.stock.proxy.mq;

import com.itcast.bull.stock.common.service.utils.BeanAssistant;
import com.itcast.bulls.stock.common.utils.GlobalConstants;
import com.itcast.bulls.stock.entity.quote.TradeStockQuoteLast;
import com.itcast.bulls.stock.proxy.netty.StockProxyServerHandler;
import com.itcast.bulls.stock.struct.netty.trade.ResponseMessage;
import com.itcast.bulls.stock.struct.netty.trade.StockHeadProto;
import com.itcast.bulls.stock.struct.netty.trade.StockQuoteProto;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.BeanUtils;
import org.springframework.cloud.stream.annotation.StreamListener;
import org.springframework.messaging.handler.annotation.Payload;
import org.springframework.stereotype.Service;

/**
 * <p>Description: </p>
 * @date
 * @author
 * @version 1.0
 * <p>Copyright:Copyright(c)2020</p>
 */
@Service
@Log4j2
public class ProxyQuoteConsumer {

    /**
     * 行情数据包自增ID（主要校验消息的连续性，同时作为消息的唯一标识）
     *
     */
    private Long seqId=1l;

    @StreamListener("quote")
    public void receiveInput1(@Payload TradeStockQuoteLast receiveMsg) {
        log.info("Process in QuoteConsumer.receiveInput1 method, receiveMsg: " + receiveMsg);
        // 1. 生成消息序列ID (实际环境中, 消息序列ID应保持唯一性, 便于追踪, 维护与排查消息)
        long stockSeqId = seqId ++;
        // 2. 构造行情包体 (用于下发给客户端使用)
        StockQuoteProto.QuoteRealtimeResp.Builder realtimeResp = StockQuoteProto.QuoteRealtimeResp.newBuilder();
        // 3. 复制行情对象属性 (从服务端的数据结构, 转换为发送给客户端的数据结构)
        BeanAssistant.copyNotNullProps(receiveMsg, realtimeResp);
        StockHeadProto.StockHead stockHead = StockHeadProto.StockHead.newBuilder()
                .setRequestType(StockHeadProto.RequestType.QUOTE_REALTIME)
                .setSeqId(stockSeqId).setStatus(GlobalConstants.STATUS_OK).build();
        ResponseMessage responseMessage = ResponseMessage.newBuilder()
                .setStockHead(stockHead)
                .setQuoteRealtimeResp(realtimeResp.build()).build();
        // 4. 消息组装处理完成之后,  广播群发行情的数据给所有登录的客户端
        StockProxyServerHandler.channels.forEach(channel -> {
            channel.writeAndFlush(responseMessage);
        });
    }
}
