package com.itcast.bulls.stock.proxy.processor;

import com.esotericsoftware.minlog.Log;
import com.itcast.bulls.stock.common.exception.ComponentException;
import com.itcast.bulls.stock.common.utils.GlobalConstants;
import com.itcast.bulls.stock.struct.grpc.trade.PlaceOrderReponse;
import com.itcast.bulls.stock.struct.grpc.trade.PlaceOrderRequest;
import com.itcast.bulls.stock.struct.grpc.trade.StockTradeDealServiceGrpc;
import com.itcast.bulls.stock.struct.netty.trade.*;
import io.netty.channel.ChannelHandlerContext;
import lombok.extern.log4j.Log4j2;
import net.devh.boot.grpc.client.inject.GrpcClient;
import org.springframework.stereotype.Service;

import javax.xml.ws.Response;

/**
 * <p>Description: </p>
 * @date
 * @author
 * @version 1.0
 * <p>Copyright:Copyright(c)2020</p>
 */
@Service
@Log4j2
public class StockDealProcessor extends AbstractStockProcessor implements IBaseStockProcessor{

    /**
     * grpc客户端接口
     */
    @GrpcClient("grpc-trade-server")
    private StockTradeDealServiceGrpc.StockTradeDealServiceBlockingStub stockTradeDealServiceBlockingStub;


    /**
     * 处理客户端请求
     * @param channelHandlerContext
     * @param stockMessage
     */
    @Override
    public void processRequest(ChannelHandlerContext channelHandlerContext, StockMessage stockMessage) {

        ResponseMessage responseMessage = null;
        StockHeadProto.StockHead stockHead = null;
        try {

            // 获取请求参数信息
            PlaceOrderData orderData = stockMessage.getPlaceOrderData();
            // 构造请求信息
            PlaceOrderRequest request = PlaceOrderRequest.newBuilder().setAccountId(getUserAccountId(channelHandlerContext))                    .setDirection(orderData.getDirection()).setExecPrice(orderData.getExecPrice())
                    .setStockId(orderData.getStockId()).setType(orderData.getType())
                    .setExecVolume(orderData.getExecVolume()).build();
            final PlaceOrderReponse response = stockTradeDealServiceBlockingStub.placeOrder(request);
            log.info("Process in StockDealProcessor.processRequest receive: " + response);
            // 返回成功报文
            stockHead = generateStockHead(stockMessage.getStockHead(), response.getStatus());
            PlaceOrderResp orderResp = PlaceOrderResp.newBuilder().setDealId(response.getDealId()).build();
            responseMessage = ResponseMessage.newBuilder().setStockHead(stockHead).setMessage(response.getMessage()).setPlaceOrderResp(orderResp).build();

        }catch(ComponentException e) {
            // 业务异常返回
            log.error(e.getMessage(), e);
            stockHead = generateStockHead(stockMessage.getStockHead(), e.getErrorCodeEnum().getCode());
            responseMessage = ResponseMessage.newBuilder().setStockHead(stockHead).setMessage(e.getErrorCodeEnum().getMessage()).build();
        }catch (Exception e) {
            // 系统异常返回
            log.error(e.getMessage(), e);
            stockHead = generateStockHead(stockMessage.getStockHead(), GlobalConstants.STATUS_ERROR);
            responseMessage = ResponseMessage.newBuilder().setStockHead(stockHead).setMessage(e.getMessage()).build();
        }

        channelHandlerContext.writeAndFlush(responseMessage);
    }

}
