package com.itcast.bulls.stock.proxy.processor;

import com.itcast.bulls.stock.common.exception.ComponentException;
import com.itcast.bulls.stock.common.exception.constants.ApplicationErrorCodeEnum;
import com.itcast.bulls.stock.struct.netty.trade.StockHeadProto;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * <p>Description: </p>
 * @date
 * @author
 * @version 1.0
 * <p>Copyright:Copyright(c)2020</p>
 */
@Component
@Log4j2
public class StockProcessorFactory {
    /**
     * 交易服务的处理器
     */
    @Autowired
    private IBaseStockProcessor stockDealProcessor;

    /**
     * 用户服务的处理器
     */
    @Autowired
    private IBaseStockProcessor stockUserProcessor;

    /**
     * 行情服务的处理器
     */
    @Autowired
    private IBaseStockProcessor stockQuoteProcessor;

    /**
     * 根据请求参数获取指定的业务处理器
     * @param requestType
     * @return
     * @throws ComponentException
     */
    public IBaseStockProcessor generateStockProcessor(StockHeadProto.RequestType requestType) throws ComponentException {

        IBaseStockProcessor processor = null;
        switch(requestType) {
            case USER_LOGIN:
                processor = stockUserProcessor;
                break;
            case PLACE_ORDER:
                processor = stockDealProcessor;
                break;
            case QUOTE_KLINE:
                processor = stockQuoteProcessor;
                break;
            default:
                log.error("Process in generateStockProcessor, not found processor! ");
                throw new ComponentException(ApplicationErrorCodeEnum.NOT_FOUND_PROCESSOR);

        }
        return processor;
    }
}
