package com.itcast.bulls.stock.proxy.test;

import com.google.protobuf.Message;
import com.google.protobuf.SourceContext;
import com.itcast.bulls.stock.struct.netty.trade.*;
import io.netty.channel.Channel;
import lombok.extern.log4j.Log4j2;

/**
 * <p>Description: </p>
 * @date
 * @author
 * @version 1.0
 * <p>Copyright:Copyright(c)2020</p>
 */
@Log4j2
public class BusinessHandler {
    private Channel ctx;

    /**
     * 消息的序列号ID
     */
    private static Long seqId = 0L;

    public BusinessHandler(Channel ctx) {
        this.ctx = ctx;
    }

    /**
     * 通过连接通道发送指定的消息
     * @param msg
     */
    private void sendMsg(Message msg) {
        if(null == ctx || null == msg || !ctx.isWritable()) {
            return;
        }
        ctx.writeAndFlush(msg);
    }

    /**
     * 下单数据接口
     */
    public void placeOrder(){
        // 下单方向
        int direct = 0;
        // 请求价格
        long execPrice = 655;
        // 成交量
        int volume = 100;
        // 股票ID
        int stockId = 1;
        // 委托类型(0:市价单, 1: 限价单)
        int type = 1;

        System.out.println("****** Send Place Order , Params ******");
        System.out.println("== direct:" + direct);
        System.out.println("== execPrice:" + execPrice);
        System.out.println("== volume:" + volume);
        System.out.println("== stockId:" + stockId);
        System.out.println("== type:" + type);

        // 构造头部消息
        StockHeadProto.StockHead stockHead = StockHeadProto.StockHead.newBuilder()
                .setSeqId(seqId++).setRequestType(StockHeadProto.RequestType.PLACE_ORDER).build();

        // 构造订单信息
        PlaceOrderData orderData = PlaceOrderData.newBuilder()
                .setDirection(direct).setExecPrice(execPrice)
                .setExecVolume(volume).setStockId(stockId).setType(type).build();
        // 组装请求消息数据包
        StockMessage stockMessage = StockMessage.newBuilder().setStockHead(stockHead)
                .setPlaceOrderData(orderData).build();
        // 发送消息至服务端
        sendMsg(stockMessage);
    }


    /**
     * 用户登录接口
     */
    public void userLogin() {
        String userNo = "admin";
        String userPwd = "admin";
        System.out.println("******* Send User Login, Parameters *******");
        System.out.println("== useNo : " + userNo );
        System.out.println("== userPwd: " + userPwd);

        // 构造头部信息
        StockHeadProto.StockHead stockHead = StockHeadProto.StockHead.newBuilder().setSeqId(seqId ++ ).setRequestType(StockHeadProto.RequestType.USER_LOGIN).build();
        // 构造用户登录请求数据包
        LoginData loginData = LoginData.newBuilder().setUserNo(userNo).setUserPwd(userPwd).build();
        // 组装请求数据包
        StockMessage stockMessage = StockMessage.newBuilder().setStockHead(stockHead).setLoginData(loginData).build();
        // 发送消息至服务端
        sendMsg(stockMessage);
    }

    /**
     * 获取K线数据
     * @param ctx
     */
    public void getKline() {

        String stockCode = "601398";
        long timeStart = 1565577100;
        long timeEnd = 1571295600;
        System.out.println("****** Send Get Kline, Parameters ****** ");
        System.out.println("== stockCode: " + stockCode);
        System.out.println("== timeStart: " + timeStart);
        System.out.println("== timeEnd: " + timeEnd);

        // 构造头部信息
        StockHeadProto.StockHead stockHead = StockHeadProto.StockHead.newBuilder().setSeqId(seqId ++).setRequestType(StockHeadProto.RequestType.QUOTE_KLINE).build();
        // 构造获取K线请求接口的数据包
        KlineData klineData = KlineData.newBuilder().setStockCode(stockCode).setTimeStart(timeStart).setTimeEnd(timeEnd).build();

        // 组装请求信息
        StockMessage stockMessage = StockMessage.newBuilder().setStockHead(stockHead).setKlineData(klineData).build();

        // 发送消息至服务端
        sendMsg(stockMessage);

    }
}
