package com.itcast.bulls.stock.quote.config;

import com.alibaba.druid.pool.DruidDataSource;
import com.itcast.bulls.stock.quote.config.sharding.DataSourceTableAlg;
import io.shardingsphere.api.config.rule.ShardingRuleConfiguration;
import io.shardingsphere.api.config.rule.TableRuleConfiguration;
import io.shardingsphere.api.config.strategy.StandardShardingStrategyConfiguration;
import io.shardingsphere.shardingjdbc.api.ShardingDataSourceFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;

import javax.sql.DataSource;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Properties;

/**
 * <p>Description: </p>
 * @date
 * @author
 * @version 1.0
 * <p>Copyright:Copyright(c)2020</p>
 */
@Configuration
public class DruidSystemDataSourceConfiguration {

    /**
     * 业务数据源
     * @return
     */
    @Bean(name = "bizSystemDataSource")
    @ConfigurationProperties(prefix = "spring.datasource.bizsystem")
    public DataSource bizSystemDataSource(){
        return new DruidDataSource();
    }

    /**
     * 交易数据源
     * @return
     */
    @Bean(name = "tradeDruidDataSource")
    @ConfigurationProperties(prefix = "spring.datasource.tradesystem")
    public DruidDataSource tradeDruidDataSource(){
        return new DruidDataSource();
    }

    /**
     * 交易数据源Sharding JDBC配置
     * @return
     */
    @Bean(name = "tradeSystemDataSource")
    @Primary
    public DataSource tradeSystemDataSource(@Autowired DruidDataSource tradeDruidDataSource) throws Exception{

        ShardingRuleConfiguration shardJdbcConfig = new ShardingRuleConfiguration();
        shardJdbcConfig.getTableRuleConfigs().add(getKLineTableRule());
        shardJdbcConfig.setDefaultDataSourceName("ds_0");

        Map<String,DataSource> dataMap = new LinkedHashMap<>() ;
        dataMap.put("ds_0",tradeDruidDataSource) ;
        // 多个分库配置
//        dataMap.put("ds_2",dataTwoSource) ;
//        dataMap.put("ds_3",dataThreeSource) ;
        Properties prop = new Properties();
        prop.put("sql.show", true);
        return ShardingDataSourceFactory.createDataSource(dataMap, shardJdbcConfig, new HashMap<>(), prop);

    }


    /**
     * K线行情表的分表配置规则
     */
    private TableRuleConfiguration getKLineTableRule() {
        TableRuleConfiguration result = new TableRuleConfiguration();
        result.setLogicTable("t_trade_stock_kline");

        //  分库配置, 代表数据库分0至2共3个库, 表t_trade_stock_kline分0至9共10个表
//        result.setActualDataNodes("ds_${0..2}.t_trade_stock_kline_${0..9}");

        result.setActualDataNodes("ds_0.t_trade_stock_kline_1,ds_0.t_trade_stock_kline_5");
        // 分库规则配置
//        result.setDatabaseShardingStrategyConfig(new StandardShardingStrategyConfiguration("dataType", new DataSourceAlg()));
        result.setTableShardingStrategyConfig(new StandardShardingStrategyConfiguration("dataType", new DataSourceTableAlg()));
        return result;
    }

}
